-- scenes/mainmenu :: the main menu of the different birb scenes

--[[
  Copyright © 2019 Kazhnuz

  Permission is hereby granted, free of charge, to any person obtaining a copy of
  this software and associated documentation files (the "Software"), to deal in
  the Software without restriction, including without limitation the rights to
  use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
  the Software, and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
  FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
  IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local Scene     = require "birb.modules.scenes"
local MainMenu  = Scene:extend()

local ListBox   = require "birb.modules.menusystem.listbox"
local Widget    = require "birb.modules.menusystem.widgets"

local SubMenuWidget = Widget.Text:extend()
local SceneWidget   = Widget.Text:extend()
local ExitWidget    = Widget.Text:extend()

function MainMenu:new()
  MainMenu.super.new(self)
  self.assets:batchImport("scenes.mainmenu.assets")

  self:addMenu("main", true)
  for i=1, 4 do
    local name = i .. "player"
    self:addSubMenu(name, name)
    if i == 1 then
      self:addScene(name, scenes.Plateformer,   "plateform", i)
    end
    self:addScene(name, scenes.MovePlayer,    "topdown", i)
    self:addScene(name, scenes.MovePlayer3D,  "topdown3D", i)
    self:addScene(name, scenes.Action3D,  "bigmap3D", i)
    if i > 1 then
      self:addScene(name, scenes.MovePlayer,    "topdown (zoom)", i, "zoom")
      self:addScene(name, scenes.MovePlayer3D,  "topdown3D (zoom)", i, "zoom")
    end
  end
  self:addSubMenu("menus", "menu")
  self:addScene("menus", scenes.Inventory, "inventory")
  self:addScene("menus", scenes.Options, "options")


  self.menusystem:setSoundFromSceneAssets("navigate")

  ExitWidget(self, "main")
  self.menusystem:switchMenu("main")
end

-- MENU FUNCTION
-- Functions that serve the handling of menus

function MainMenu:addMenu(name, nobackbutton)
  local screenHeight, screenWidth = core.screen:getDimensions()
  local w, h = 424/2, 240 - 48
  local x, y = w / 2, 24
  ListBox(self.menusystem, name, x, y, w, h, 8)
end

function MainMenu:addSubMenu(name, fullname)
  self:addMenu(name)
  SubMenuWidget(self, "main", name, fullname)
  SubMenuWidget(self, name, "main", "back", 1)
end

function MainMenu:addScene(submenu, scene, fullname, arg1, arg2, arg3, arg4, arg5)
  local args = {arg1, arg2, arg3, arg4, arg5}
  SceneWidget(self, submenu, scene, fullname, args)
end

function MainMenu:draw()
  love.graphics.setColor(.3,  .1, .4, 1)
  love.graphics.rectangle("fill", 0, 0, 424, 240)
  utils.graphics.resetColor()
  love.graphics.print(math.floor(game.playtime), 8, 8)
end

-- WIDGETS
-- Widgets used by menus

-- Submenu widget :: go to a submenu

function SubMenuWidget:new(scene, menu, newmenu, fullname, order)
  self.scene = scene
  local widgetmenu  = self.scene.menusystem.menus[menu]
  local font        = self.scene.assets.fonts["medium"]
  self.newmenu      = newmenu
  local label = ""
  if fullname == "back" then
    label       = core.lang:translate("commons", fullname)
    self.sfx    = "cancel"
  else
    label       = core.lang:translate("mainmenu", fullname)
    self.sfx    = "confirm"
  end
  SubMenuWidget.super.new(self, widgetmenu, font, label)
  self.order = order or 0
end

function SubMenuWidget:action()
  self.scene.assets:playSFX(self.sfx)
  self.scene.menusystem:switchMenu(self.newmenu)
end

-- Scene widget :: switch scene

function SceneWidget:new(scene, menu, newscene, fullname, args)
  self.scene  = scene
  self.args   = args
  local widgetmenu  = self.scene.menusystem.menus[menu]
  local font        = self.scene.assets.fonts["medium"]
  self.newscene     = newscene
  local label       = core.lang:translate("mainmenu", fullname)
  SceneWidget.super.new(self, widgetmenu, font, label)
end

function SceneWidget:action()
  core.scenemanager:storeCurrentScene("mainmenu")
  core.scenemanager:prepareTransition()
  self.newscene(self.args[1], self.args[2], self.args[3], self.args[4], self.args[5])
end

-- Exit Widget : exit the scenes

function ExitWidget:new(scene, menu)
  self.scene = scene
  local widgetmenu  = self.scene.menusystem.menus[menu]
  local font        = self.scene.assets.fonts["medium"]
  local label       = core.lang:translate("commons", "exit")
  SceneWidget.super.new(self, widgetmenu, font, label)
end

function ExitWidget:action()
  game:write()
  love.event.quit()
end

return MainMenu
